// cIOLine.cc -- an abstract class for handling I/O lines.
// Copyright (C) 1995 Ari Mujunen. (amn@nfra.nl, Ari.Mujunen@hut.fi)

// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License.
// See the file 'COPYING' for details.

// $Log: cIOLine.cc,v $
// Revision 1.4  1995/06/18  15:10:19  amn
// Oops, 'open()' didn't close the old 'fileDesc' first.
//
// Revision 1.3  1995/06/16  15:09:48  amn
// '.h' -> '.hh' for C++ headers.
//
// Revision 1.2  1995/03/30  11:08:51  amn
// 'close()' is successful when it returns ==0 (not != -1).
//
// Revision 1.1  1995/02/11  19:41:43  amn
// Initial revision
//

#include "registration.h"
RCSID(cIOLine_cc, "$Id: cIOLine.cc,v 1.4 1995/06/18 15:10:19 amn Exp $");


#pragma implementation
#include "cIOLine.hh"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <assert.h>


void
cIOLine::open(const char *pathName)
{
  if (fileDesc != -1) {
    close();
  }
  assert((fileDesc = ::open(pathName, O_RDWR | O_NOCTTY)) != -1);
    // As long as we rely on termios' cc[VTIME] timeout mechanism,
    // 'O_NONBLOCK' should not be specified.
  rememberedPathname = pathName;
  defaultSettings();
}  // open

// This finalizes line settings just after opening a file descriptor.
// Most inheritors will want to supply a version that more accurately
// sets the required defaults.
void
cIOLine::defaultSettings(void)
{
}  // defaultSettings

void
cIOLine::close(void)
{
  int retCodeFromClose;

  if (fileDesc == -1) {
    return;
  }
  assert((retCodeFromClose = ::close(fileDesc)) == 0);
  fileDesc = -1;
  rememberedPathname = "";
}  // close
