// dbtest -- a program to test MkIII data buffer interface class
//           'cDataBufferLine'.
// Copyright (C) 1995 Ari Mujunen. (amn@nfra.nl, Ari.Mujunen@hut.fi)

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program -- see the file 'COPYING'; if not, write to
// the Free Software Foundation, Inc.,
// 675 Mass Ave, Cambridge, MA 02139, USA.

// $Log: dbtest.cc,v $
// Revision 1.3  1995/07/11  17:29:43  amn
// Memory test diag output, first tests about split mode and sync word
// detection.
//
// Revision 1.2  1995/06/25  15:30:24  amn
// Initial tests at Westerbork with 9600 baud.
//
// Revision 1.1  1995/06/16  15:20:41  amn
// Initial revision
//


#include "registration.h"
RCSID(test_cc, "$Id: dbtest.cc,v 1.3 1995/07/11 17:29:43 amn Exp amn $");


// This class we use directly, so we need its header.
#include "cDataBufferLine.hh"

// C++ I/O library.
#include <iostream.h>

// printf().
#include <stdio.h>

// exit().
#include <stdlib.h>


static int
countBits(int x)
{
  int y;
  
  // xxx: might want to replace with table lookup
  y = x & 0x01;
  y += ((x >>= 1) & 0x01);
  y += ((x >>= 1) & 0x01);
  y += ((x >>= 1) & 0x01);
  y += ((x >>= 1) & 0x01);
  y += ((x >>= 1) & 0x01);
  y += ((x >>= 1) & 0x01);
  y += ((x >>= 1) & 0x01);
  return y;
}  // countBits

static void
findSyncWordStart(tBlock& aBlock,
		  int& bitIndex,
		  int thresholdNumberOfBits = 36
		  )
{
  // Readily-shifted sync word (and mask) patterns.
  static const unsigned char sw[8][6] = {
    { 0xff, 0x7f, 0xbf, 0xdf, 0xe0, 0x00 },
    { 0x7f, 0xbf, 0xdf, 0xef, 0xf0, 0x00 },
    { 0x3f, 0xdf, 0xef, 0xf7, 0xf8, 0x00 },
    { 0x1f, 0xef, 0xf7, 0xfb, 0xfc, 0x00 },
    { 0x0f, 0xf7, 0xfb, 0xfd, 0xfe, 0x00 },
    { 0x07, 0xfb, 0xfd, 0xfe, 0xff, 0x00 },
    { 0x03, 0xfb, 0xfd, 0xff, 0x7f, 0x80 },
    { 0x01, 0xfe, 0xff, 0x7f, 0xbf, 0xc0 }
  };
  static const unsigned char sm[8][6] = {
    { 0xff, 0xff, 0xff, 0xff, 0xf0, 0x00 },
    { 0x7f, 0xff, 0xff, 0xff, 0xf8, 0x00 },
    { 0x3f, 0xff, 0xff, 0xff, 0xfc, 0x00 },
    { 0x1f, 0xff, 0xff, 0xff, 0xfe, 0x00 },
    { 0x0f, 0xff, 0xff, 0xff, 0xff, 0x00 },
    { 0x07, 0xff, 0xff, 0xff, 0xff, 0x80 },
    { 0x03, 0xff, 0xff, 0xff, 0xff, 0xc0 },
    { 0x01, 0xff, 0xff, 0xff, 0xff, 0xe0 }
  };
  int j;

  // Starting at bit 'bitIndex' to search for sync word (4.5 bytes).
  for (j=(bitIndex / 8); j<507; j++) {
    int b;
    
    for (b = (j==0 ? (bitIndex % 8) : 0); b < 8; b++) {
      int numberOfMatchingBits;
      int n;

      numberOfMatchingBits = 0;
      for (n=0; n<6; n++) {
	numberOfMatchingBits += countBits((aBlock[j+n] ^ ~(sw[b][n])) & sm[b][n]);
	// xxx: of course the constants in 'sw' need to be negated if this works...
      }
      if (numberOfMatchingBits >= thresholdNumberOfBits) {
	// xxx: this is a match
	bitIndex = (j * 8) + b;
	return;
      }
    }  // for each bit
  }  // for each byte
  // Oops, not a single match good enough found.
  bitIndex = -1;
  return;
}  // findSyncWordStart


typedef enum {
  ePPEven = 0,
  ePPOdd
} tParityPolarity;

// 
static int
checkAndRemoveParityBits(tBlock& aBlock,
			 int bitIndex,
			 tParityPolarity pol = ePPOdd)
{
  int err = 0;
  int x,y,i;

  y = 0;
  i = 7-(bitIndex % 8);
  for (x=0; x < 511; x++) {
    unsigned short int t = ((aBlock[x]<<8) | aBlock[x+1]) >> i;
      
    if (t & 0x01 != pol) {
      err++;
    }
    aBlock[y++] = (unsigned char)(t >> 1);
    i--;
    if (i<0) {
      x++;  // xxx
      i = 7;
    }
  }  // for each byte
  return err;
}  // checkAndRemoveParityBits

int
main(
  int argc,
  char *argv[]
)
{
  cDataBufferLine dbLine;
  
  cout << "Starting 'dbtest'...\n";
  if (argc > 1) {
    dbLine.open(argv[1]);
  } else {
    dbLine.open("/dev/ttyS1");
  }

  // xxx: Remember 115200 vs. 9600 baud...

  dbLine.hello(eDataSourceInternal);

#define MEMORY_TEST 1
#if MEMORY_TEST
  // Definitely not for memory test: dbLine.setMode(eModeSplit);
  {
    int tn;
    int i;

    for (tn=0; tn <= 11; tn++) {
      int errorsPerWholeBuffer;
      int maintainZeroAddressBits;
      int maintainOneAddressBits;
      tCheckInfo ci;
      tCheckInfo ciPrevErr;

      cout << "Starting memory test #" << tn << "...";
      cout.flush();

      dbLine.reset();
      dbLine.loadTest(tn, 100);

      errorsPerWholeBuffer = 0;
      maintainZeroAddressBits = maintainOneAddressBits = 0;
      ciPrevErr.clearCheckInfo();  // initially "previous error" == "no error"
      for (i=0; i<255; i++) {
        dbLine.checkTestBlock(i, ci, /*suppressMessages=>*/ 1, stdout);
	if (ci.errors) {
	  errorsPerWholeBuffer += ci.errors;
          maintainZeroAddressBits |= (ci.maintainZeroAddressBits | (i << 9));
          maintainOneAddressBits |= (ci.maintainOneAddressBits | (0x1fe00 & ~(i << 9)));
	  if (ci != ciPrevErr) {
	    // New kind of error block, store it and
	    // report it by checking again with messages turned on.
	    cout << "\n";
	    ciPrevErr = ci;
	    dbLine.checkTestBlock(i, ci, /*suppressMessages=>*/ 0, stdout);
	  } else {  // same kind of error again
	    static int firstTimeDuringOneRun = 1;

	    if (firstTimeDuringOneRun) {
	      cout << "# ('#' means 'repeat the previous error message') ";
	      firstTimeDuringOneRun = 0;
	    } else {
	      cout << "#";
	    }
	  }
	} else {  // no errors in this block
	  cout << ".";
	}
        cout.flush();
      }  // for each block in buffer
      if (errorsPerWholeBuffer) {
	cout << " " << errorsPerWholeBuffer << " error" << (errorsPerWholeBuffer==1 ? "" : "s") << " with this test pattern.\n";
	cout << "For these error conditions the address lines in schematic 6004-003";
	// Detect the case when all address lines bits were indeterminate
 	// (all were both 0 and 1 during error bytes.)
	if ((maintainZeroAddressBits & maintainOneAddressBits & 0x1ffff) == 0x1ffff) {
	  cout << "\nall address lines had both zero and one bits on them.\n";
	} else {
	  int mzab;
	  int moab;
	  int bit;

	  cout << " were:\n";
	  mzab = maintainZeroAddressBits;
	  moab = maintainOneAddressBits;
	  for (bit=0; bit <= 16; bit++) {  // 17bits, 128kByte
	    if (mzab & moab & 0x01) {
	      // If both bits are on, we don't really know if the address
	      // line is always 0 or 1.
	    } else {
	      if (bit == 0) {
		cout << "  HIBYTE was always ";
	      } else {
		printf("     A%02d was always ", bit-1);
	      }
	      if (!(mzab & 0x01)) {
		cout << "zero; ";
		if (!(moab & 0x01)) {
		  cout << "one; cannot be at the same time!?!";
		}
		cout << "\n";
	      } else {
		if (!(moab & 0x01)) {
		  cout << "one;\n";
		} else {
		  // Print nothing since the bit seems indeterminate.
		}
	      }
	    }  // else this bit must be always 0 or 1
	    
	    // And to next bit...
	    mzab >>= 1;
	    moab >>= 1;
	  }  // for each data buffer address bit 0..16

	  {  // Determine memory bank guesses.
	    int bank;
	    static char sch[4][9] = {
	      "6004-009",
	      "6004-010",
	      "6004-011",
	      "6004-012"
	    };
	  
	    if (((maintainZeroAddressBits ^ maintainOneAddressBits) & 0x18000) == 0x18000) {
	      // A single memory bank.
	      cout << "If the fault is in memory chips (and not in buses),\nit might be isolated in memory bank ";
	      bank = (maintainZeroAddressBits & 0x18000) >> 15;
	      cout << bank << " (see schematic " << sch[bank] << ").\n";
	    } else if ((maintainZeroAddressBits & maintainOneAddressBits & 0x18000) == 0x18000) {
	      // Not much point declaring that the fault can be in any bank...
	    } else {
	      // Two memory banks, should be added some day...  xxx:
	    }
	  }  // block of mem bank guesses
	}  // else there are bits which are always 0/1
	cout << "\n";
      } else {
	cout << " no errors with this test pattern.\n";
      }
    }  // for each error pattern
  }
  dbLine.readStatisticsToStderr();
  exit(0);
#endif

  cout << "Arming the data buffer...";
  cout.flush();
  dbLine.reset();
  //dbLine.setMode(eModeSplit);
  dbLine.setMode(eModeDecoderA);
  dbLine.armAndLoad(1200);
  {
    int errorCount=0;
    int i;
    tBlock aBlock;

#define COUNT(x) if (x != eDataBufferOk) errorCount++

    for (i=70; i<82; i++) {
      dbLine.sendData(aBlock);
      COUNT(dbLine.getStatus());
#if 0
      aBlock[511] = 0;
      printf("%s\n", aBlock);
#endif
#if 1
      { int j;
	int bitIndex;

        printf("Block %03d: ", i);
	bitIndex = 0;
	while (1) {
	  findSyncWordStart(aBlock, bitIndex, 35);
	  if (bitIndex < 0) {
	    break;
	  }
	  printf("(sync word found at bit %d)\n", bitIndex);
	  checkAndRemoveParityBits(aBlock,
				   bitIndex,
				   ePPOdd);
  
	  // Continue searching from next byte...
	  bitIndex += 8;
	}  // while
        for (j=0; j<512; j++) {
          printf("%02X ", aBlock[j]);
        }
        printf("\n\n");
      }
#endif
    }  // for

    dbLine.readStatisticsToStderr();

    cout << "\n";
    cout << "Total data buffer errors = " << errorCount << "\n";
    cout << "\n";
  }
  dbLine.goodbye();

  dbLine.close();
  cout << "'dbtest' finished.\n";
}  // main
